#define uchar uint8_t

#include "onewire.h"

#if defined(__ets__)
    #include <Libraries/OneWire/OneWire.h>
#endif

#include "DS18X20.h"
#include "crc8.h"

/* *************************************************************************/

#if defined(__AVR__)
void address_dallas(uint8_t dallas_idx) {
#endif

#if defined(__ets__)
void address_dallas(uint8_t dallas_idx, OneWire W1_bus) {
#endif
    #if defined(__AVR__)
    uint8_t i;
	ow_reset();
	ow_byte_wr(OW_MATCH_ROM);
	for (i = 0; i < 8; i++) {
	    ow_byte_wr(sensor_addreses[dallas_idx][i]);
				}
    #endif
    #if defined(__ets__)
	W1_bus.reset();
	W1_bus.select(&sensor_addreses[dallas_idx][0]);
    #endif
					}

/* *************************************************************************/
/* should return ZERO if CRC is correct */

#if defined(__AVR__)
uint8_t read_dallas(uint8_t sensor_idx) {
#endif

#if defined(__ets__)
uint8_t read_dallas(uint8_t sensor_idx, OneWire W1_bus) {
#endif

#if defined(__AVR__)
    uint8_t i;
	ow_byte_wr(DS18X20_READ_SCRATCHPAD);
	    for (i = 0; i < 9; i++) {
		scratchpad_table[sensor_idx][i] = ow_byte_rd();
				    }
#endif
#if defined(__ets__)
	W1_bus.write(DS18X20_READ_SCRATCHPAD);
	W1_bus.read_bytes(&scratchpad_table[sensor_idx][0], 9);
#endif
	return crc8(&scratchpad_table[sensor_idx][0],9);
							}

/* *************************************************************************/
/* decodes scratchpads into temperature sign/value/decimals */

void decode_dallas(uint8_t sensor_idx) {
	/* decode Temp, depending on sensor family */
	temp_sign[sensor_idx] = (scratchpad_table[sensor_idx][1] & 0xF0) >> 4;
	switch (sensor_addreses[sensor_idx][0]) {
	    case 0x28: /* DS18B20 12bit model */
		if (temp_sign[sensor_idx] != 0) {
			      temp_units[sensor_idx] = ((~scratchpad_table[sensor_idx][0] >> 4) & 0x0F) | ((~scratchpad_table[sensor_idx][1] & 0x07) << 4);
			      temp_parts[sensor_idx] = 625 * ((~scratchpad_table[sensor_idx][0] & 0x0F) + 1); }
		    else	{ /* positive */
			      temp_units[sensor_idx] = (scratchpad_table[sensor_idx][0] >> 4) | ((scratchpad_table[sensor_idx][1] & 0x07) << 4);
			      temp_parts[sensor_idx] = 625 * (scratchpad_table[sensor_idx][0] & 0x0F);
				}
		break;
	    case 0x10: /* DS1820 OLD model */
		if (temp_sign[sensor_idx] == 0) {
		    temp_units[sensor_idx] = ((scratchpad_table[sensor_idx][0] >> 1) & 0x7F);
		    temp_parts[sensor_idx] = 5000 * (scratchpad_table[sensor_idx][0] & 0x01);
						}
		else { // NEGATIVE
		    temp_units[sensor_idx] = ((~scratchpad_table[sensor_idx][0] >> 1) & 0x7F);
		    temp_parts[sensor_idx] = 5000 * (scratchpad_table[sensor_idx][0] & 0x01);
		     }
		break;
						} /* switch */
	temp_100s[sensor_idx] = (uint16_t)temp_units[sensor_idx] * 100 + (temp_parts[sensor_idx] / 100);
	if ((temp_parts[sensor_idx] % 100) > 50) { temp_100s[sensor_idx]++; }
					}

/* *************************************************************************/

