/* Name: main.c
 (
 * board wirings for "Arduino" board, Mega328P
 *
 * PB5 	 - RED LED - to GND.
 *
 * 
 * PD0	 - RxD - header to USB ->serial converter.
 * PD1	-  TxD - header to USB ->serial converter.
 *
 * PB2	 - (D10) - Dallas with 4k7 pull-up.
 
 * PD0-3 - (D4-D7) LCD data D4-D7
 * PD3   - (D3) - LCD RS
 * PB0	 - (D2) - LCD RW
 * PD6	 - (D8) - LCD E
 *
 * PC0-5 - (A0-A5) - pushbuttons
 *
 * FUSE BITS (328p):
 * LFUSE - E7 - 231
 * HFUSE - D1 - 209
 * EFUSE - 07 - 7
 *
 * AUTHOR (C) laurentp@wp.pl
 *
 * License: GPL2+
 *
 * 2013-10-26 - main program finished, missing only error handling,
 *		what is to do: configuration menu, serial connection.
 *		just added CRC over scratchpad (works!), with signalling dead sensor.
 *		have to add "delta T" measurement (?).
 * 2013-11-05 - everything works, configuration code is ugly, but works.
 *
 * 2013-11-07 - have to add serial comm.
 * 2013-12-10 - serial transmit works, 
 * 2013-12-12 - added pushbutton on INT0/PD2, as on PD7 does not work (LCD lib?)
 * 		so now have to add "delta" for "head" sensor?
 * 2013-12-23 - corrected displaying of 2nd digit in temp_units -> was 07.00 is  7.00
 * 2014-01-02 - changed descriptions of T's (doesn't matter for 4 line, but may for 2 line LCD)
 * 		added "buzzer_mask" byte -> each bit will be 0.1s beep, 0.2s at end always silent
 *
 * 2014-02-09 - killed bug in ioinit procedures causing PD7 incorrect behaviour, 
 *		also removed code to init LCD pins -> let the library handle it.
 *
 * 2014-02-21 - PAY ATTENTION TO CLOCK SOURCE!
 *
 *		moved code from PORTx / PINx to KEY_x / LED_x
 *		have to improve display procedures
 *
 * 2014-03-15 - deltas done
 *
 * 2016-04-12 -
 * 		programming: 1). set HFUSE to D2, (D4?) (D1 for NO bootloader) (209)
 *				 set LFUSE to EF (FF), (239/255)
				 set EFUSE to 07,
 *				 set LOCK  to 3F,
 *			     2). flash Optiboot (my version( using USBasp,
 *				 flash EEPROM using USBasp,
 *			     3). put processor in target board, press and hold S2,
 *			         power the board, 
 *			         flash program using "Arduino" / /dev/ttyUSB0
 *
 * 2014-04-11 -
 *		wrote keys filtering (but not quadrature encoder),
 *		added "press & hold" for KEY_3 -> column "ALARM MODE",
 *		added "press & hold" for KEY_4,
 *		added commas on serial.
 *		temperature on serial is always absolute -> column for now
 *
 * 2014-04-13 -
 *		correct buzzer control (patterns), 
 *		moved key sensing in CONFIG mode to soft keys
 * 		moving to store temperature labels in EEPROM, but for now
 *			we need an external ISP / program, to SET EEPROM (bootloader does not support)
 *
 * 2014-04-17 -
 (		begin removing "PANMUFINEK" specific mods from code, leaving only different EEPROM contents
 *		
 * 2014-05-04 -
 * 		added second alarm (head), cleaned code which handles errors (--.--), 
 * 		rewrite of some printing / displaying code.
 * 
 * 2014-05-20 -
 *		added "preheat" column alarm (50 < 70 deg, delta@10sec > 5deg, until cancel)
 *
 * 		CODING STYLE CLEANUP NEEDED!!!
 * 2015-12-09 -
 *		Started porting program to Arduino Nano
 *
 * 2015-12-25 -
 *		Porting done, program works on:
 *		- "production board" - schematics & PCB by laurentp, 
 *		- using Mega 8 OR Mega328P (need to recompile), 
 *		- "Arduino Nano dev board" - using Mega328P.
 *		Switch is done by changing board type in "defines.h" and/or
 *		processor type in "Makefile"
 *
 * 2015-12-28 -
 *		Started porting to ESP8266 / Sming platform.
 *
 * 2016-01-03 -
 *		Program compiles for ESP8266 / Sming, shows some "I am alive"
 * 2016-01-06 -
 *		Program works correctly on ESP, with config, WiFi, WiFi config.
 * 2016-01-11 -
 *		TCP server almost done (working, but have to manage more than 1 client case
 *
 * 2016-04-12 -
 *		added 60*C alarm for Twody (unconditional, AVR for now?)
 *		first implementation of rotary encoder (AVR)
 *		changed LCD update to 100ms (was 500ms) to reflect encoder state changes in "more real" time
 *
 * 2016-05-06 -
 *		Akas table (Tliquid to liquid% lookup) done and working, 
 *		Timer working "most of cases", added +/x rotating symbol in 1st char of 3rd line if running
 *		Added possibility to display running time in 3rd line
 *
 * 2016-11-20 -
 *		Added more steps to TIME_TABLE (for timer), added TIME_TABLE & PWM_TABLE size definitions
 *		in defines.h
 *		try to use HardwareTimer (precise) on ESP8266 fail, got cashes from time to time, could not accept.
 *		HX711 test fails - can't get 0.01*C stable reading, next try is STS21 (on ESP for now)
 *		Added PWM output instead of key 5 (at 3rd line of LCD on "production" board).
 *		granulatity 1-2-3-4-5-6-7-8-9-10-20-30-40-50-60-70-80-90-100%, 
 *		1-10% in 100s period, 10-100% in 10s period, for electric valve control.
 *		Column alarm and Timer alarm disable PWM output.
 * 2016-11-27 -
 *		Corrected temperature decode for NEGATIVE temperatures.
 *		Corrected negative temperature display.
 *
 * 2017-01-28 -
 *		Started removing 2 line display support.
 *
 * 2017-05-14 -
 *		Working support for STS21 sensor on I2C, for ESP for now, need to compare with DS.
 *		implemented SENSIRION's CRC8 calculation.
 *		Working I2C IO for AVR "board by krzyski"
 */

#if defined(__AVR__)
    #include <avr/io.h>
    #include <avr/interrupt.h>
    #include <avr/pgmspace.h>
    #include <avr/eeprom.h>
    #include <avr/wdt.h>
    #include <avr/sleep.h>
#endif

#include "defines.h"

#if defined(__AVR__)
    #include <util/delay.h>
    #include <util/setbaud.h>
#endif

/* pay attention to LCD_START_LINEx in *.h */
    #include "lcd.h"
#if defined(__ets__)
    #include <Libraries/LiquidCrystal/LiquidCrystal_I2C.h>
    #include <SmingCore/FileSystem.h>

    #if defined(NODEMCU_HARDWARE_TIMER)
	#include <SmingCore/HardwareTimer.h>
    #endif
#endif

#if defined(__AVR__)
    #include <string.h> 
    #include <stdlib.h>
#endif

/* laurent - Dallas */
    #include "onewire.h"
    #include "crc8.h"
#if defined(__ets__)
    #include <Libraries/OneWire/OneWire.h>
#endif

#if !(defined(__AVR__) || defined(__ets__))
    #error("Architecture NOT supported")
#endif

    #include "ioinit.h"
    #include "timer_int.h"
    #include "ones_count.h"
    #include "virtual_lcd.h"
    #include "DS18X20.h"
    #include "do_config.h"
    #include "do_1s_stuff.h"

#if defined(__AVR__)
    #include "uart_int.h"
#endif

#if !defined(MAIN_H)
    #include "main.h"
#endif

/* ******************************************************************** */
/* ESP8266 quick & dirty code partitioning                              */
/* ******************************************************************** */

#if defined(__ets__)
    #include "main_tcp_server.h"
    #include "main_http.cpp"
#endif

/* Laurent's begin */
#define uchar uint8_t
/* set Timer1 to 10ms		*/
const 	 	uint16_t Timer1_prescale = Timer1_prescale_default;
/* half period (10ms!) */
const uint16_t half_period = 0.01 * (F_CPU / Timer1_prescale_default);

/* **********************************************************************/
/* disable watchdog 	MUST BE for newer Megas				*/
/* **********************************************************************/
#if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
__attribute__((naked, section(".init3"))) void get_mcusr(void)
{
/*    mcusr_mirror = MCUSR; */
    MCUSR = 0;
    wdt_disable();
}
#endif

/* ------------------------------------------ */
/* prepares (decodes) byte into buffer_tx, as decimal no 100's (for time)  */
/* ------------------------------------------ */
#if defined INCLUDE_RTC
void prepare_time_to_display(void)
{
	lcd_time[0] = (hour / 10) + '0';
	lcd_time[1] = (hour % 10) + '0';
	lcd_time[2] = ':';
	lcd_time[3] = (minute / 10) + '0';
	lcd_time[4] = (minute % 10) + '0';
	lcd_time[5] = ':';
	lcd_time[6] = (second / 10) + '0';
	lcd_time[7] = (second % 10) + '0';
	lcd_time[8] = ',';
}
#endif

/* ------------------------------------------ */
/* same for downcount TIMER */
/* ------------------------------------------ */
#if defined DOWNCOUNT_TIMER
void prepare_downcount_timer_to_display(void)
{
	lcd_time[0] = (downcount_timer_hour % 10) + '0';
	lcd_time[1] = ':';
	lcd_time[2] = (downcount_timer_minute / 10) + '0';
	lcd_time[3] = (downcount_timer_minute % 10) + '0';
	lcd_time[4] = ':';
	lcd_time[5] = (downcount_timer_second / 10) + '0';
	lcd_time[6] = (downcount_timer_second % 10) + '0';
}
#endif

/* -------------------------------------------- */

void Send_Initial_text(void)
{
#if defined(__AVR__)
/* wait until previous buffer is sent */
      while ((flagi & FLAG_SERIAL_TX_TO_SEND) != 0) { sleep_mode(); }
      strcpy_P((char *)&buffer_tx, (PGM_P)&Initial_text);

/* arming serial Tx  */
	flagi |= FLAG_SERIAL_TX_TO_SEND;			/* buffer_tx to send */
	tx_pointer = 0;			/* reset pointer */
    #if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
	UCSR0B |= _BV(UDRIE0);		/* ARM Tx buffer empty interrupt */
    #endif
    #if defined(__AVR_ATmega8__)
	UCSRB |= _BV(UDRIE);		/* ARM Tx buffer empty interrupt */
    #endif
    /* wait to complete */
	while ((flagi & FLAG_SERIAL_TX_TO_SEND) != 0) { sleep_mode(); }
#endif

#if defined(__ets__)
    Serial.print(Initial_text);
#endif
}

/* *************************************************************************/

uint16_t Akas_lookup(uint16_t temp)
    {
    uint32_t	percent_4;
    uint16_t	percent_1, percent_2, percent_3;
    uint8_t		temp_unit, temp_part;

	temp_unit = (temp / 100);							// 92*C
        temp_part = (temp % 100);							//  0.25*C
#if defined(__AVR__)
	percent_1 = pgm_read_word((PGM_P)&tabela_akasa[100 - temp_unit][1]);	// 13.00%, 9
	percent_2 = pgm_read_word((PGM_P)&tabela_akasa[100 - temp_unit - 1][1]);	// 11.00%
#endif
#if defined(__ets__)
	percent_1 = tabela_akasa[100 - temp_unit][1];
	percent_2 = tabela_akasa[100 - temp_unit - 1][1];
#endif
	percent_3 = (percent_1 - percent_2);					// 2.00%
	percent_4 = ((uint32_t)temp_part * (uint32_t)percent_3) / 100; 		// 0.50%
	return(percent_1 - (uint16_t)percent_4); 					//12.75%
}

/* *************************************************************************/

void	display_3rd_lcd_line(void) {
	    /* 1st char of 3rd line - x/+ if Timer is running */
	      if ((flagi3 & DOWNCOUNT_TIMER_STARTED_FLAG) != 0) {
			switch (virt_lcd_line2[16]) {
			    case ' ': virt_lcd_line2[16] = '+';
				      break;
			    case '+': virt_lcd_line2[16] = 'x';
				      break;
			    case 'x': virt_lcd_line2[16] = '+';
				      break;
						    }
								}
	      else { virt_lcd_line2[16] = ' '; }

	      switch (lcd_third_line_mode)  {
	    /* 3 - 3rd line as PWM output */
		case 3: 	virt_lcd_line2[17] = 'P';
				virt_lcd_line2[18] = 'W';
				virt_lcd_line2[19] = 'M';
				virt_lcd_line2[20] = ' ';
				virt_lcd_line2[21] = 'o';
				virt_lcd_line2[22] = 'u';
				virt_lcd_line2[23] = 't';
				virt_lcd_line2[24] = '=';
				if (pwm_output == 100) {
				    virt_lcd_line2[25] = '1';
				    virt_lcd_line2[26] = '0';
							}
				else {
				    virt_lcd_line2[25] = ' ';
				    if (((pwm_output % 100) / 10) > 0)  {
					virt_lcd_line2[26] = ((pwm_output % 100) / 10) + '0';
									}
				    else {
					virt_lcd_line2[26] = ' ';
				         }
				     }
				virt_lcd_line2[27] = (pwm_output % 10) + '0';
				virt_lcd_line2[28] = '%';
				virt_lcd_line2[29] = ' ';
				if ((flagi3 & PWM_ENABLED_FLAG) != 0) {
				    virt_lcd_line2[30] = 'E';
				    virt_lcd_line2[31] = 'N';
								      }
				else {
				    virt_lcd_line2[30] = ' ';
				    virt_lcd_line2[31] = ' ';
				     }
				break;

	    /* 2 - 3rd line as UPTIME */
		case 2: 	virt_lcd_line2[17] = 'U';
				virt_lcd_line2[18] = 'p';
				virt_lcd_line2[19] = 'T';
				virt_lcd_line2[20] = 'i';
				virt_lcd_line2[21] = 'm';
				virt_lcd_line2[22] = 'e';
				virt_lcd_line2[23] = '=';
				memcpy(&virt_lcd_line2[24], &lcd_time[0], 8);
				break;

	    /* 1 - 3rd line as downcount TIMER */
		case 1: 	virt_lcd_line2[17] = ' ';
				virt_lcd_line2[18] = ' ';
				virt_lcd_line2[19] = 'T';
				virt_lcd_line2[20] = 'i';
				virt_lcd_line2[21] = 'm';
				virt_lcd_line2[22] = 'e';
				virt_lcd_line2[23] = 'r';
				virt_lcd_line2[24] = '=';
				prepare_downcount_timer_to_display();
				memcpy(&virt_lcd_line2[25], &lcd_time[0], 7);
				break;

		case 0:		/* restore Twody text */
			        memcpy(&virt_lcd_line2[17], &temperature_labels[2][0], 7);
				virt_lcd_line2[24] = '=';
				memcpy(&virt_lcd_line2[25], &buffer_tx[52], 7);
				break;
					    }
	   } /* display_3rd_lcd_line */

/* *************************************************************************/

#if defined(__AVR__)
int main(void)
#endif
#if defined(__ets__)
void init(void)
#endif
{
    ioinit();

#if defined(__ets__)
//    System.setCpuFrequency(eCF_160MHz);

    Serial.systemDebugOutput(false);

    num_of_sensors = 4;

    lcd_flagi = LCD_4LINE_FLAG;

file_t config_file;

    config_file = fileOpen("sensors_addresses.bin", eFO_ReadOnly);
    for (i = 0; i < num_of_sensors; i++) {
	fileRead(config_file, &sensor_addreses[i][0], 8);
			    }
    fileClose(config_file);
    config_file = fileOpen("sensors_labels.bin", eFO_ReadOnly);
    for (i = 0; i < num_of_sensors; i++) {
	fileRead(config_file, &temperature_labels[i][0], 8);
			    }
    fileClose(config_file);
#endif

#if defined(__AVR__)
    num_of_sensors = 4;
    lcd_flagi = LCD_4LINE_FLAG;
    //eeprom_read_byte(&lcd_flags_in_eeprom);
    for (i = 0; i < num_of_sensors; i++) {
	for (j = 0; j < 8; j++) {
	    sensor_addreses[i][j] = eeprom_read_byte(&sensor_addreses_eeprom[i][j]);
				}
					 }
    /* read temperature labels from EEPROM */
    for (i = 0; i < num_of_sensors; i++) {
	for (j = 0; j < 7; j++) {
	    temperature_labels[i][j] = eeprom_read_byte(&temperature_labels_eeprom[i][j]);
				}
			    }
#endif
    /* CLS virtual display */
    for (i = 0; i < 48; i++) { virt_lcd_line2[i] = 0x20; }
    for (i = 0; i < 16; i++) { lcd_line1[i] = 0x20; }
    /* copy labels to (virtual) display */
    memcpy(&lcd_line1[1], &temperature_labels[0][0], 7);
    memcpy(&virt_lcd_line2[1], &temperature_labels[1][0], 7);
    memcpy(&virt_lcd_line2[17], &temperature_labels[2][0], 7);
    memcpy(&virt_lcd_line2[33], &temperature_labels[3][0], 7);

    lcd_line1[0] = 'T';
    lcd_line1[8] = '=';
    virt_lcd_line2[0] = 'T';
    for (i = 0; i < 3; i++) { virt_lcd_line2[(16 * i) + 8] = '='; }

/* DEBUG */
    for (i = 0; i < sizeof(buffer_tx); i++) { buffer_tx[i] = ' '; }
    buffer_tx[sizeof(buffer_tx) - 3] = '\r';
    buffer_tx[sizeof(buffer_tx) - 2] = '\n';
    buffer_tx[sizeof(buffer_tx) - 1] = 0;
/* /DEBUG */

    /* switch off LEDs */
#if defined(PRODUCTION_BOARD) || defined(PRODUCTION_BOARD_BY_KRZYSKI)
    LED_1_PORT |= _BV(LED_1_BIT);
#endif
#if defined(DEV_BOARD_NANO)
    LED_1_PORT &= ~_BV(LED_1_BIT);
#endif
#if defined(DEV_BOARD_NODE_MCU)
    digitalWrite(LED_1_PIN, HIGH);
#endif

    Send_Initial_text();

/* start up WiFi / TCP-IP stuff */
#if defined(__ets__)
	AppSettings.load();
	WifiStation.enable(true);
	if (AppSettings.exist()) {
		WifiStation.config(AppSettings.ssid, AppSettings.password);
		if (!AppSettings.dhcp && !AppSettings.ip.isNull())
			WifiStation.setIP(AppSettings.ip, AppSettings.netmask, AppSettings.gateway);
				 }
	WifiStation.startScan(networkScanCompleted);
	WifiAccessPoint.enable(false);
	WifiStation.waitConnection(connectOk, 30, connectFail);
#endif

// LCD start up
#if defined(__AVR__)
    lcd_init(LCD_DISP_ON);
    #if defined(DEFINE_DEGREES_SIGN)
	    lcd_def_cgram_char(0x08, 0x08, 0x14, 0x08, 0x06, 0x08, 0x08, 0x06, 0x00); /* define degrees sign as CGRAM(1) */
	    lcd_line1[15] = 1;
	    virt_lcd_line2[15] = 1;
	    virt_lcd_line2[31] = 1;
	    virt_lcd_line2[47] = 1;
    #endif
#endif

/* if KEY_1/2 pressed during power-up, enter CONFIG mode */
#if defined(__AVR__)
    #if defined(KEY_1_BIT)
	if ((KEY_1_PINR & _BV(KEY_1_BIT)) == 0) { flagi |= IN_CONFIG_FLAG; } // Production / Nano
    #else
	if ((KEY_2_PINR & _BV(KEY_2_BIT)) == 0) { flagi |= IN_CONFIG_FLAG; } // Production by Krzyski
    #endif
#endif

#if defined(__ets__)
    Wire.pins(I2C_SCL, I2C_SDA);
    lcd.begin(16,4);
    #if defined(KEY_1_PIN)
	if (digitalRead(KEY_1_PIN) == 0) { flagi |= IN_CONFIG_FLAG; }
    #else
	if (digitalRead(KEY_2_PIN) == 0) { flagi |= IN_CONFIG_FLAG; }
    #endif
#endif

    if ((num_of_sensors == 0) || (num_of_sensors == 0xFF)) { flagi |= IN_CONFIG_FLAG; }

    if (( flagi & IN_CONFIG_FLAG ) == 0) { /* normal work */

	flagi3 |= TEMP_1_FILTER_ENABLED;
	flagi3 |= TEMP_2_FILTER_ENABLED;

/***************************************************************************/
/*                             main AVR program loop                       */
/***************************************************************************/
#if defined(__AVR__)
    for(;;) { /* main event loop */
	sleep_mode();

	if ((flagi & MS100_FLAG) != 0) {
	    flagi &= ~MS100_FLAG;
				       } /* 100ms test */

/* ------------------------------------------ */
/* 500ms test */
	if ((flagi & MS500_FLAG) != 0) {
	    flagi &= ~MS500_FLAG;
	    /* LCD horizontal scroll*/
	    if (lcd_horiz_offset < 48)  { lcd_horiz_offset++; }
		else			{ lcd_horiz_offset = 0; }
	    LCD_virt_display();
				       } /* 500ms test */

/* ------------------------------------------ */
/* 1s */
	if ((flagi & S1_FLAG) != 0) {
	    flagi &= ~S1_FLAG;
	    do_1s_stuff();
				      } /* /S1_FLAG test */

	    } /* neverending for */
#endif // defined(__AVR__)

	} /* IF !IN_CONFIG */
	else { do_config(); }
#if defined(__AVR__)
    return 0;
#endif
#if defined(__ets__)
    return;
#endif

} /* main() / init() */

/* END PROGRAM */
