/* 2015-12-27: 	compiles under ESP8266 environement */
/*		KEY_x / LED_x definitions NOT yet ported to ESP */

/* defining basic constraints for project */
#define DEFINES_H

#include <inttypes.h>

#if defined(__AVR__)
    /* CPU frequency */
    #define F_CPU 16000000UL
#endif

/* UART baud rate */
#if defined(__AVR__)
    #define UART_BAUD 38400U
#endif
#if defined(__ets__)
    #define UART_BAUD 115200U
#endif
#define BAUD UART_BAUD

/* Timer 1 prescaler 10ms @ 16MHz */
#define Timer1_prescale_default 64U

#define INCLUDE_RTC
#define DOWNCOUNT_TIMER

#define AUTO_PWM_DOWNCOUNT_TIMER

#define PWM_TIMER

#define COLUMN_ALARM_DELTA_FILTER_ENABLED
#define HEAD_ALARM_DELTA_FILTER_ENABLED

#if defined(__AVR__)
//	#define PRODUCTION_BOARD /* ATMega8 / ATMega328P, board by laurentp@wp.pl */
	#define PRODUCTION_BOARD_BY_KRZYSKI /* ATMega328P, board by krzyskiba@gmail.com */
	#define I2C_IO			    /* IO on I2C PCF8574A expander */
	#define I2C_IO_ADDRESS 0x4E
//    #define DEV_BOARD_NANO /* handwired Arduino Nano board */

#endif

#if defined(__ets__)
    #define DEV_BOARD_NODE_MCU /* handwired ESP8266/NodeMCU board, LCD on I2C expander */
//    #define I2C_IO			    /* IO on I2C PCF8574A expander */
//    #define I2C_IO_ADDRESS 0x4E
    #define	I2C_STS_ADDRESS	0x4A // 0x94?
    #define NODEMCU_HARDWARE_TIMER
#endif

//#define HEARTBEAT_LED
/* #define PANMUFINEK */
//#define DEFINE_DEGREES_SIGN

#define HEAD_ALARM_TYPE				0x01
#define COLUMN_ALARM_TYPE			0x02
#define PREHEAT_COLUMN_ALARM_TYPE		0x04
#define WATER_ALARM_TYPE			0x08
#define TIMER_ALARM_TYPE			0x10
#define	POGON_ALARM_TYPE			0x20

/* mask definitions for flagi4 byte (program flags) */
#define TEMP_1_FILTER_SATURATED			0x01
#define TEMP_1_FILTER_SATURATED_VARIABLE	flagi4
#define TEMP_2_FILTER_SATURATED			0x02
#define TEMP_2_FILTER_SATURATED_VARIABLE	flagi4

/* mask definitions for flagi3 byte (program flags) */
#define DOWNCOUNT_TIMER_STARTED_FLAG		0x80
#define PREHEAT_COLUMN_ALARM_CANCEL		0x40
#define	PWM_ENABLED_FLAG			0x20
#define	COLUMN_STABILIZED_FLAG			0x10
#define	TEMP_1_FILTER_ENABLED			0x08
#define	TEMP_2_FILTER_ENABLED			0x04
#define	AUTO_PWM_DOWNCOUNT_TIMER_STARTED_FLAG	0x02
#define	AUTO_PWM_STARTED_FLAG			0x01

/* mask definitions for flagi2 byte (program flags) */
#define COLUMN_IS_DELTA_FLAG			0x80
#define COLUMN_IS_DELTA_FLAG_VARIABLE		flagi2
#define HEAD_IS_DELTA_FLAG			0x40
#define HEAD_IS_DELTA_FLAG_VARIABLE		flagi2
#define LIQUID_AS_PERCENT_FLAG			0x20
#define LIQUID_AS_PERCENT_FLAG_VARIABLE		flagi2
#define	ALARM_1_CLEARED_FLAG			0x10
#define	ALARM_2_CLEARED_FLAG			0x08
#define	ALARM_3_CLEARED_FLAG			0x04
#define	ALARM_4_CLEARED_FLAG			0x02
#define	COLUMN_ALARM_ENABLED_FLAG		0x01
#define	COLUMN_ALARM_ENABLED_FLAG_VARIABLE	flagi2

/* mask definitions for flagi byte (program flags) */
#define	HEAD_ALARM_ENABLED_FLAG			0x80
#define	HEAD_ALARM_ENABLED_FLAG_VARIABLE	flagi
#define MS500_FLAG				0x40
#define S1_FLAG					0x20
#define	FLAG_SERIAL_RX_DATA			0x10
#define	FLAG_SERIAL_TX_TO_SEND			0x08
#define MS100_FLAG 				0x04
#define	CONFIG_CLEARED_FLAG			0x02
#define IN_CONFIG_FLAG				0x01

#define	SOFT_KEY_1_MASK			0x01
#define	SOFT_KEY_1_MASK_VARIABLE	soft_keys
#define	SOFT_KEY_2_MASK			0x02
#define	SOFT_KEY_2_MASK_VARIABLE	soft_keys
#define	SOFT_KEY_3_MASK			0x04
#define	SOFT_KEY_4_MASK			0x08
#define	SOFT_KEY_5_MASK			0x10
#define	SOFT_KEY_6_MASK			0x20
#define	SOFT_KEY_6_MASK_VARIABLE	soft_keys
#define SOFT_KEY_3_OLD_MASK		0x40
#define SOFT_KEY_4_OLD_MASK		0x80

/* in SOFT_KEYS2 */
#define	SOFT_KEY_1_OLD_MASK		0x01
#define	SOFT_KEY_1_OLD_MASK_VARIABLE	soft_keys2
#define	SOFT_KEY_2_OLD_MASK		0x02
#define	SOFT_KEY_2_OLD_MASK_VARIABLE	soft_keys2
#define	SOFT_KEY_5_OLD_MASK		0x04
#define	SOFT_KEY_6_OLD_MASK		0x08
#define	SOFT_KEY_6_OLD_MASK_VARIABLE	soft_keys2
#define	SOFT_ENCODER_MINUS_MASK		0x10
#define	SOFT_ENCODER_MINUS_OLD_MASK	0x20
#define	SOFT_ENCODER_PLUS_MASK		0x40
#define	SOFT_ENCODER_PLUS_OLD_MASK	0x80

/* in SOFT_KEYS3 */
#define	SOFT_ENCODER_SWITCH_MASK	0x01
#define	SOFT_ENCODER_SWITCH_OLD_MASK	0x02

#define PWM_TABLE_MAX			20
#define MIN_AUTO_PWM			10

#define AUTO_PWM_RELOAD_MINUTE		10
#define	AUTO_PWM_RELOAD_SECOND		0

#define TIME_TABLE_MAX			18

#if defined PRODUCTION_BOARD 
    #define	KEY_1_PORT			PORTC
    #define	KEY_1_PINR			PINC
    #define	KEY_1_BIT			PC4
    #define	KEY_2_PORT			PORTC
    #define	KEY_2_PINR			PINC
    #define	KEY_2_BIT			PC0
    #define	KEY_3_PORT			PORTB
    #define	KEY_3_PINR			PINB
    #define	KEY_3_BIT			PB5
    #define	KEY_4_PORT			PORTB
    #define	KEY_4_PINR			PINB
    #define	KEY_4_BIT			PB4

    #define	KEY_5_PORT			PORTB
    #define	KEY_5_PINR			PINB
    #define	KEY_5_BIT			PB3
    #define	KEY_5_DDR			DDRB

    #define	KEY_6_PORT			PORTB
    #define	KEY_6_PINR			PINB
    #define	KEY_6_BIT			PB2

    #define	ENCODER_PLUS_PORT		PORTC
    #define	ENCODER_PLUS_PINR		PINC
    #define	ENCODER_PLUS_BIT		PC3
    #define	ENCODER_MINUS_PORT		PORTC
    #define	ENCODER_MINUS_PINR		PINC
    #define	ENCODER_MINUS_BIT		PC1
    #define	ENCODER_SWITCH_PORT		PORTC
    #define	ENCODER_SWITCH_PINR		PINC
    #define	ENCODER_SWITCH_BIT		PC2
    /* LED / buzzer definitions */
    #define	LED_1_PORT			PORTB
    #define	LED_1_BIT			PB1
    #define	LED_1_DDR			DDRB
#endif

#if defined PRODUCTION_BOARD_BY_KRZYSKI

// VALID

    #define	KEY_2_PORT			PORTC
    #define	KEY_2_PINR			PINC
    #define	KEY_2_BIT			PC0

    #define	KEY_3_PORT			PORTB
    #define	KEY_3_PINR			PINB
    #define	KEY_3_BIT			PB5

    #define	KEY_4_PORT			PORTB
    #define	KEY_4_PINR			PINB
    #define	KEY_4_BIT			PB4

#if !defined(I2C_IO)
    #define	KEY_5_PORT			PORTC
    #define	KEY_5_PINR			PINC
    #define	KEY_5_BIT			PC5
    #define	KEY_5_DDR			DDRC
#endif
    #define	KEY_6_PORT			PORTB
    #define	KEY_6_PINR			PINB
    #define	KEY_6_BIT			PB2

    #define	ENCODER_PLUS_PORT		PORTC
    #define	ENCODER_PLUS_PINR		PINC
    #define	ENCODER_PLUS_BIT		PC3

    #define	ENCODER_MINUS_PORT		PORTC
    #define	ENCODER_MINUS_PINR		PINC
    #define	ENCODER_MINUS_BIT		PC1

    #define	ENCODER_SWITCH_PORT		PORTC
    #define	ENCODER_SWITCH_PINR		PINC
    #define	ENCODER_SWITCH_BIT		PC2
    /* LED / buzzer definitions */
    #define	LED_1_PORT			PORTB
    #define	LED_1_BIT			PB3
    #define	LED_1_DDR			DDRB
#endif

#if defined DEV_BOARD_NANO
    #define	KEY_1_PORT			PORTC
    #define	KEY_1_PINR			PINC
    #define	KEY_1_BIT			PC0
    #define	KEY_2_PORT			PORTC
    #define	KEY_2_PINR			PINC
    #define	KEY_2_BIT			PC1
    #define	KEY_3_PORT			PORTC
    #define	KEY_3_PINR			PINC
    #define	KEY_3_BIT			PC2
    #define	KEY_4_PORT			PORTC
    #define	KEY_4_PINR			PINC
    #define	KEY_4_BIT			PC3

    #define	KEY_5_PORT			PORTB
    #define	KEY_5_PINR			PINB
    #define	KEY_5_BIT			PB3
    #define	KEY_5_DDR			DDRB

    #define	KEY_6_PORT			PORTB
    #define	KEY_6_PINR			PINB
    #define	KEY_6_BIT			PB4

    #define	ENCODER_PLUS_PORT		PORTC
    #define	ENCODER_PLUS_PINR		PINC
    #define	ENCODER_PLUS_BIT		PC4
    #define	ENCODER_MINUS_PORT		PORTC
    #define	ENCODER_MINUS_PINR		PINC
    #define	ENCODER_MINUS_BIT		PC5
    #define	ENCODER_SWITCH_PORT		PORTB
    #define	ENCODER_SWITCH_PINR		PINB
    #define	ENCODER_SWITCH_BIT		PB1

    /* LED / buzzer definitions */
    #define	LED_1_PORT			PORTB
    #define	LED_1_BIT			PB5
    #define	LED_1_DDR			DDRB
#endif

#if defined DEV_BOARD_NODE_MCU
//    #define	KEY_5_PIN			9
//    #define	KEY_1_PIN			14
    #define	KEY_2_PIN			13
    #define	KEY_3_PIN			12
    #define	KEY_4_PIN			14

    /* LED / buzzer definitions */
    #define	LED_1_PIN			2
#endif

/* mask definitions for flags in eeprom byte */
#define	LCD_OPERATION_MODE_FLAG		0x80
#define	LCD_4LINE_FLAG			0x40

#define LCD_DEFAULT_FLAGS		LCD_4LINE_FLAG

/* error codes definitions */
#define	EEPROM_DATA_CORRUPTED		1
#define NOT_IMPLEMENTED			2

/*-------------------------------------------------------------*/

#if defined PRODUCTION_BOARD
    #define LCD_PORT         PORTD        /**< port for the LCD lines   */
    #define LCD_DATA0_PORT   LCD_PORT     /**< port for 4bit data bit 0 */
    #define LCD_DATA1_PORT   LCD_PORT     /**< port for 4bit data bit 1 */
    #define LCD_DATA2_PORT   LCD_PORT     /**< port for 4bit data bit 2 */
    #define LCD_DATA3_PORT   LCD_PORT     /**< port for 4bit data bit 3 */
    #define LCD_DATA0_PIN    5            /**< pin for 4bit data bit 0  */
    #define LCD_DATA1_PIN    4            /**< pin for 4bit data bit 1  */
    #define LCD_DATA2_PIN    3            /**< pin for 4bit data bit 2  */
    #define LCD_DATA3_PIN    2            /**< pin for 4bit data bit 3  */
    #define LCD_RS_PORT      PORTB        /**< port for RS line         */
    #define LCD_RS_PIN       0            /**< pin  for RS line         */
    #define LCD_RW_PORT      PORTD        /**< port for RW line         */
    #define LCD_RW_PIN       7            /**< pin  for RW line         */
    #define LCD_E_PORT       PORTD        /**< port for Enable line     */
    #define LCD_E_PIN        6            /**< pin  for Enable line     */
#endif

/*-------------------------------------------------------------*/

#if defined PRODUCTION_BOARD_BY_KRZYSKI
    #define LCD_PORT         PORTD        /**< port for the LCD lines   */
    #define LCD_DATA0_PORT   LCD_PORT     /**< port for 4bit data bit 0 */
    #define LCD_DATA1_PORT   LCD_PORT     /**< port for 4bit data bit 1 */
    #define LCD_DATA2_PORT   LCD_PORT     /**< port for 4bit data bit 2 */
    #define LCD_DATA3_PORT   LCD_PORT     /**< port for 4bit data bit 3 */
    #define LCD_DATA0_PIN    5            /**< pin for 4bit data bit 0  */
    #define LCD_DATA1_PIN    4            /**< pin for 4bit data bit 1  */
    #define LCD_DATA2_PIN    3            /**< pin for 4bit data bit 2  */
    #define LCD_DATA3_PIN    2            /**< pin for 4bit data bit 3  */
    #define LCD_RS_PORT      PORTB        /**< port for RS line         */
    #define LCD_RS_PIN       0            /**< pin  for RS line         */
    #define LCD_RW_PORT      PORTD        /**< port for RW line         */
    #define LCD_RW_PIN       7            /**< pin  for RW line         */
    #define LCD_E_PORT       PORTD        /**< port for Enable line     */
    #define LCD_E_PIN        6            /**< pin  for Enable line     */
#endif

/*-------------------------------------------------------------*/

#if defined DEV_BOARD_NANO
    #define LCD_PORT         PORTD        /**< port for the LCD lines   */
    #define LCD_DATA0_PORT   LCD_PORT     /**< port for 4bit data bit 0 */
    #define LCD_DATA1_PORT   LCD_PORT     /**< port for 4bit data bit 1 */
    #define LCD_DATA2_PORT   LCD_PORT     /**< port for 4bit data bit 2 */
    #define LCD_DATA3_PORT   LCD_PORT     /**< port for 4bit data bit 3 */
    #define LCD_DATA0_PIN    4            /**< pin for 4bit data bit 0  */
    #define LCD_DATA1_PIN    5            /**< pin for 4bit data bit 1  */
    #define LCD_DATA2_PIN    6            /**< pin for 4bit data bit 2  */
    #define LCD_DATA3_PIN    7            /**< pin for 4bit data bit 3  */
    #define LCD_RS_PORT      PORTD        /**< port for RS line         */
    #define LCD_RS_PIN       2            /**< pin  for RS line         */
    #define LCD_RW_PORT      PORTB        /**< port for RW line         */
    #define LCD_RW_PIN       0            /**< pin  for RW line         */
    #define LCD_E_PORT       PORTD        /**< port for Enable line     */
    #define LCD_E_PIN        3            /**< pin  for Enable line     */
#endif

#if defined(DEV_BOARD_NODE_MCU)
    #define		LCD_TYPE	0 // I2C
//	#define	LCD_TYPE	1 // SR
    // I2C GPIO pin no
    #define		I2C_SDA		4
    #define		I2C_SCL		0
    #define		I2C_LCD_ADDR	0x27
    // LCD pins on I/O expander PCF8754
    #define 		LCD_E		2
    #define		LCD_RW		1
    #define		LCD_RS		0
    #define		LCD_BACKLIGHT	3

    #define		LCD_D4		4
    #define		LCD_D5		5
    #define		LCD_D6		6
    #define		LCD_D7		7
#endif
/*-------------------------------------------------------------*/

#if defined PRODUCTION_BOARD
    #define OW_PIN  PC5
    #define OW_IN   PINC
    #define OW_OUT  PORTC
    #define OW_DDR  DDRC
#endif

#if defined PRODUCTION_BOARD_BY_KRZYSKI
    #define OW_PIN  PB1
    #define OW_IN   PINB
    #define OW_OUT  PORTB
    #define OW_DDR  DDRB
#endif

#if defined DEV_BOARD_NANO
    #define OW_PIN  PB2
    #define OW_IN   PINB
    #define OW_OUT  PORTB
    #define OW_DDR  DDRB
#endif

#if defined(DEV_BOARD_NODE_MCU)
    // GPIO 5
    #define OW_PIN  5
#endif
/*-------------------------------------------------------------*/
