/*******************************************************/
// main module, to be invoked from MAIN program on AVR, 
// from 500ms interrupt on ESP
/*******************************************************/

#include "defines.h"
#include <stdlib.h>
#include "DS18X20.h"
#include "crc8.h"
#include "do_1s_stuff.h"

#if defined(__AVR__)
    #include <avr/pgmspace.h>
#endif

void do_1s_stuff(void) {
//    #if defined(I2C_IO)
//	i2c_start(I2C_IO_ADDRESS);
//	if ((second & 0x01) == 0) { i2c_write(0x00); }
//	else			  { i2c_write(0xFF); }
//	i2c_stop();
//    #endif

	    prepare_time_to_display();
	    memcpy(&buffer_tx[0], &lcd_time[0], 9);
	    buffer_tx[8] = ',';
	    /* read Dallas */
	    lcd_pointer = 0;
	    for (j = 0; j < num_of_sensors; j++) {
#if defined(__AVR__)
		address_dallas(j);
		error_code = read_dallas(j);
#endif
#if defined(__ets__)
	if ((STS_Ready) && (j == 0)) {
		STS_Temperature = ((float)((STS_byte1 << 8) + STS_byte2) * 175.72 / (1 << 16)) - 46.85;
		error_code = sensirion_crc8(&STS_byte1, 3);
				     }
	else 		{ /* try to read Dallas */
		address_dallas(j, OneWire_bus);
		error_code = read_dallas(j, OneWire_bus);
			}
#endif
		if (error_code == 0) {
	#if defined(__ets__)
		    temp_100s_valid[j] = true;
		if ((j == 0) && (STS_Ready)) {
		/* decode STS_temperature back to 100's - Dallas format */
			temp_100s[j] = (uint16_t)(STS_Temperature * 100);
			temp_units[j] = (uint8_t)(temp_100s[j] / 100);
			temp_parts[j] = (uint8_t)(temp_100s[j] % 100);
		/* disable filter for STS */
			flagi3 &= ~TEMP_1_FILTER_ENABLED;
					     }
		else	{
		    decode_dallas(j);
			}
	#endif
	#if defined(__AVR__)
		    temp_100s_valid |= ( 1 << j);
		    decode_dallas(j);
	#endif
		    /* display temperature on VIRTUAL display*/
		    temp_units_t = temp_units[j];
		    temp_parts_t = temp_parts[j];
		    temp_sign_t = temp_sign[j];
		    temp_100s_t = temp_100s[j];

		    /* walking filter to detect deltas / average on column sensor */
		    if (j == 0) { temp_1_filter_10sec_100s[temp_1_filter_10sec_pointer++] = temp_100s_t;
				  if (temp_1_filter_10sec_pointer == 10) { temp_1_filter_10sec_pointer = 0; }
				  temp_100s_t2 = 0;
				  temp_1_filtered_10sec_100s = 0;
				  temp_1_filtered_10sec_100s_32 = 0;
				  temp_100s_t2_min = 10000;
				  temp_100s_t2_max = 0;
				  for (i = 0; i < 10; i++) {	temp_100s_t2 = temp_1_filter_10sec_100s[i];
								if (temp_100s_t2 < temp_100s_t2_min) { temp_100s_t2_min = temp_100s_t2; }
								if (temp_100s_t2 > temp_100s_t2_max) { temp_100s_t2_max = temp_100s_t2; }
								temp_1_filtered_10sec_100s_32 += (uint32_t)temp_1_filter_10sec_100s[i];
							    }
				  temp_1_filtered_10sec_100s_32 /= 10;
				  temp_1_filtered_10sec_100s = (uint16_t)temp_1_filtered_10sec_100s_32;
				  if ((temp_100s_t > 5000) && 
				      (temp_100s_t < 7000) && 
				      ((flagi3 & PREHEAT_COLUMN_ALARM_CANCEL) == 0) &&
				      ((temp_100s_t2_max - temp_100s_t2_min) > 300)) { alarm_type |= PREHEAT_COLUMN_ALARM_TYPE; }
				  if ((flagi3 & PREHEAT_COLUMN_ALARM_CANCEL) != 0)   { alarm_type &= ~PREHEAT_COLUMN_ALARM_TYPE; }

				  if ((flagi3 & TEMP_1_FILTER_ENABLED) != 0) {
					temp_100s_t = temp_1_filtered_10sec_100s;
									     }
				}
		    /* END walking filter to detect deltas / average on column sensor */

		    /* walking filter to detect deltas / average on head sensor */
		    if (j == 1) { temp_2_filter_10sec_100s[temp_2_filter_10sec_pointer++] = temp_100s_t;
				  if (temp_2_filter_10sec_pointer == 10) { temp_2_filter_10sec_pointer = 0; }
				  temp_2_filtered_10sec_100s_32 = 0;
				  temp_2_filtered_10sec_100s = 0;
				  for (i = 0; i < 10; i++) {	temp_100s_t2 = temp_2_filter_10sec_100s[i];
								temp_2_filtered_10sec_100s_32 += (uint32_t)temp_100s_t2;
							   }
				  temp_2_filtered_10sec_100s_32 /= 10;
				  temp_2_filtered_10sec_100s = (uint16_t)temp_2_filtered_10sec_100s_32;

				  if ((flagi3 & TEMP_2_FILTER_ENABLED) != 0) {
					temp_100s_t = temp_2_filtered_10sec_100s;
									     }
				}
		    /* END walking filter to average on head sensor */

		    /* always calcuate delta */
		    if (temp_100s_t >= temp_delta_base_100s[j]) {
		        temp_100s_d = temp_100s_t - temp_delta_base_100s[j];
		        temp_sign_d = 0;		   }
		    else {
		        temp_100s_d = temp_delta_base_100s[j] - temp_100s_t;
		        temp_sign_d = 1;		   }

		    temp_100s_delta[j] = temp_100s_d;
		#if defined(__AVR__)
		    if (temp_sign_d == 0) { temp_100s_delta_sign &= ~(1 << j); }
		    else 		  { temp_100s_delta_sign |= (1 << j); }
		#endif
		#if defined(__ets__)
		    temp_100s_delta_sign[j] = temp_sign_d;
		#endif

		    if ((j == 0) || (j == 1)) { buffer_tx[(j * 17) + 9] = 'T'; }
		    else 		      { buffer_tx[(j * 17) + 9] = ' '; }
		    /* copy labels */
		    memcpy(&buffer_tx[(j * 17) + 10],&temperature_labels[j][0], 7);
		    buffer_tx[(j * 17) + 17] = '=';
		    buffer_tx[(j * 17) + 21] = '.';
#if defined(DEFINE_DEGREES_SIGN)
		    buffer_tx[(j * 17) + 24] = 'C';
#else
		    buffer_tx[(j * 17) + 24] = 'C';
#endif

/* DEBUG */
//    #if defined(__ets__)
//    if (j == 0) {   lcd_pointer = 1;
		    //prepare_hex_byte_to_lcd1(STS_byte1);
		    //prepare_hex_byte_to_lcd1(STS_byte2);
		    //prepare_hex_byte_to_lcd1(sensirion_crc8(&STS_byte1, 3));
//		    prepare_hex_byte_to_lcd1(STS_byte3);
//		}
//    #endif
/* /DEBUG */
		    buffer_tx[(j * 17) + 25] = ',';

		    switch (j) {
		        case 0:	
				/* print (possibly) 100's */
				if (temp_100s_t > 10000)    	{ buffer_tx[(j * 17) + 18] = '1'; }
				    else			{ buffer_tx[(j * 17) + 18] = ' '; }

				if (temp_sign_t != 0)		{ buffer_tx[(j * 17) + 18] = '-'; }

				if (temp_100s_t < 1000)  	{ buffer_tx[(j * 17) + 19] = ' '; }
				    else		 	{ buffer_tx[(j * 17) + 19] = ( temp_100s_t % 10000) / 1000 + '0'; }
				buffer_tx[(j * 17) +20] = (temp_100s_t  % 1000) / 100  + '0';
				buffer_tx[(j * 17) +22] = (temp_100s_t % 100) / 10 + '0';
				switch (sensor_addreses[j][0]) {
				    case 0x28: /* DS18B20 12bit model */
					buffer_tx[(j * 17) +23] = (temp_100s_t % 10) + '0';
					break;
				    case 0x10: /* OLD 1820 0.5deg resolution */
					buffer_tx[(j * 17) +23] = '-';
					break;
								}
				/* FIRST digit (100's) / SIGN */
				if ((flagi2 & COLUMN_IS_DELTA_FLAG) != 0) {
				/* for deltas display sign */
				    if (temp_sign_d == 0)     { lcd_line1[9] = '+'; }
					else		      { lcd_line1[9] = '-'; }

				    if (temp_100s_d < 1000)  { lcd_line1[10] = ' '; }
						      else   { lcd_line1[10] = (temp_100s_d % 10000) / 1000 + '0'; }

				    lcd_line1[11] = (temp_100s_d  % 1000) / 100  + '0';
				    lcd_line1[13] = (temp_100s_d % 100) / 10 + '0';

				    switch (sensor_addreses[j][0]) {
					case 0x28: /* DS18B20 12bit model */
					    lcd_line1[14] = (temp_100s_d % 10) + '0';
					    break;
					case 0x10: /* OLD 1820 0.5deg resolution */
					    lcd_line1[14] = '-';
					    break;
								    }

									  } /* COLUMN_IS_DELTA != 0  */
				else { /* COLUMN_IS_DELTA == 0  */
				       /* work is already done for printing, so don't do twice */
				       memcpy(&lcd_line1[0], &buffer_tx[(17 * j) + 9], 16);
				     } /* COLUMN_IS_DELTA == 0  */

				if ((flagi2 & COLUMN_ALARM_ENABLED_FLAG) != 0) {
				    lcd_line1[0] = 'A';
				    if (temp_100s_d > 15)	{ alarm_type |= COLUMN_ALARM_TYPE;
								  flagi3 &= ~PWM_ENABLED_FLAG; }
									       }
			/* AUTOMATIC MODE ENABLED */
				if ((flagi3 & AUTO_PWM_STARTED_FLAG) !=0) {
				/* COLUMN is NOT stabilized */
				    if ((flagi3 & COLUMN_STABILIZED_FLAG) == 0) {
				    /* NEGATIVE or ZERO delta */
					if ((temp_sign_d == 1) || 
					    ((temp_sign_d == 0) && (temp_100s_d == 0))) {
						flagi3 |= COLUMN_STABILIZED_FLAG;
						virt_lcd_line2[32] = 'S';
						/* no AUTO timer running - start PWM */
						if ((flagi3 & AUTO_PWM_DOWNCOUNT_TIMER_STARTED_FLAG) == 0)  {
						    flagi3 |= PWM_ENABLED_FLAG;
													    }
											}
										 } /* COLUMN_STABILIZED */
				/* COLUMN is stabilized */
				    else 	{
				    /* POSITIVE delta > 0.15*C */
					    if ((temp_sign_d == 0) && (temp_100s_d > 15)) {
						flagi3 &= ~COLUMN_STABILIZED_FLAG;
						virt_lcd_line2[32] = 'U';
						flagi3 &= ~PWM_ENABLED_FLAG;
						auto_pwm_downcount_timer_minute = AUTO_PWM_RELOAD_MINUTE;
						auto_pwm_downcount_timer_second = AUTO_PWM_RELOAD_SECOND;
						flagi3 |= AUTO_PWM_DOWNCOUNT_TIMER_STARTED_FLAG;
						pwm_output_idx--;
				#if defined(__AVR__)
						pwm_output = pgm_read_byte((PGM_P)&PWM_table[pwm_output_idx]);
				#endif
				#if defined(__ets__)
						pwm_output = PWM_table[pwm_output_idx];
						#endif
						/* end AUTO mode on PWM <= 10% */
						if (pwm_output <= MIN_AUTO_PWM) {
						    alarm_type = COLUMN_ALARM_TYPE;
						    flagi3 &= ~AUTO_PWM_STARTED_FLAG;
						    virt_lcd_line2[32] = ' ';
								      }
											  }
						}
									  }
			    break;

/********************** 2nd line **************************************************/
			    case 1: /* initialize output 2nd line */
				    if ((temp_100s_t / 10000) > 0)  { buffer_tx[(17 * j) + 18] = '1'; }
						else		    { buffer_tx[(17 * j) + 18] = ' '; }

				    if (temp_sign_t != 0)		{ buffer_tx[(j * 17) + 18] = '-'; }

				    if (temp_100s_t < 1000)  	{ buffer_tx[(17 * j) + 19] = ' '; }
				      else		 	{ buffer_tx[(17 * j) + 19] = ( temp_100s_t % 10000) / 1000 + '0'; }
				    buffer_tx[(17 * j) + 20] = (temp_100s_t  % 1000) / 100  + '0';
				    buffer_tx[(17 * j) + 22] = (temp_100s_t % 100) / 10 + '0';
				    switch (sensor_addreses[j][0]) {
					case 0x28: /* DS18B20 12bit model */
					    buffer_tx[(17 * j) + 23] = (temp_100s_t % 10) + '0';
					    break;
					case 0x10: /* OLD 1820 0.5deg resolution */
					    buffer_tx[(17 * j) + 23] = '-';
					    break;
								   }
				    /* finished printing, now displaying */
				    if ((flagi2 & HEAD_IS_DELTA_FLAG) != 0) {
				    /* for deltas display sign */
				    if (temp_sign_d == 0)     { virt_lcd_line2[9] = '+'; }
				      else		      { virt_lcd_line2[9] = '-'; }
				
				    if (temp_100s_d <  1000) 		{ virt_lcd_line2[10] = ' '; }
					else 				{ virt_lcd_line2[10] = (temp_100s_d % 10000) / 1000 + '0'; }
					
				    virt_lcd_line2[11] = (temp_100s_d % 1000) / 100 + '0';
				    virt_lcd_line2[13] = ((temp_100s_d % 100) / 10)+ '0';	
				    switch (sensor_addreses[j][0]) {
					    case 0x28: /* DS18B20 12bit model */
						virt_lcd_line2[14] = (temp_100s_d % 10) + '0';
						break;
					    case 0x10: /* OLD 1820 model */
						virt_lcd_line2[14] = '-';
						break;
								      }
									    }
				    else {  /* HEAD_IS_DELTA_FLAG == 0 */
				       /* work is already done for printing, so don't do twice */
				       memcpy(&virt_lcd_line2[0], &buffer_tx[26], 16);

				         }
				if ((flagi & HEAD_ALARM_ENABLED_FLAG) != 0) {
				    virt_lcd_line2[0] = 'A';
				    if (temp_100s_d > 10)	{ alarm_type |= HEAD_ALARM_TYPE; }
									    }
				break;

/********************** 3rd line **************************************************/
			    case 2: 
				    /* unconditional alarm on Tpary(wody) > 60 degC */
				        if (temp_units_t > 59)  { alarm_type |= WATER_ALARM_TYPE; }
						else 		{ alarm_type &= ~WATER_ALARM_TYPE; }
				    /* / end alarm > 60 */
					if ((temp_100s_t / 10000) > 0)  { buffer_tx[18 + (17 * j)] = '1'; }
					    else			{ buffer_tx[18 + (17 * j)] = ' '; }

					if (temp_sign_t != 0)		{ buffer_tx[(j * 17) + 18] = '-'; }

					if (temp_100s_t <  1000) { buffer_tx[19 + (17 * j)] = ' '; }
					else {			   buffer_tx[19 + (17 * j)] = (temp_100s_t % 10000) / 1000 + '0';
					     }
					buffer_tx[20 + (17 * j)] = (temp_100s_t % 1000) / 100 + '0';
					buffer_tx[22 + (17 * j)] = ((temp_100s_t % 100) / 10)+ '0';
					switch (sensor_addreses[j][0]) {
					    case 0x28: /* DS18B20 12bit model */
						buffer_tx[23 + (17 * j)] = (temp_100s_t % 10) + '0';
						break;
					    case 0x10: /* OLD 1820 model */
						buffer_tx[23 + (17 * j)] = '-';
						break;
									}
					display_3rd_lcd_line();
				    break;

/***************************************8 4th line ***********************************************/
			    case 3:
				        if ((temp_100s_t / 10000) > 0)  { buffer_tx[18 + (17 * j)] = '1'; }
					    else			{ buffer_tx[18 + (17 * j)] = ' '; }

					if (temp_sign_t != 0)		{ buffer_tx[(j * 17) + 18] = '-'; }

					if (temp_100s_t <  1000) { buffer_tx[19 + (17 * j)] = ' '; }
					    else {		   buffer_tx[19 + (17 * j)] = (temp_100s_t % 10000) / 1000 + '0'; }
					buffer_tx[20 + (17 * j)] = (temp_100s_t % 1000) / 100 + '0';
					buffer_tx[22 + (17 * j)] = ((temp_100s_t % 100) / 10)+ '0';
					switch (sensor_addreses[j][0]) {
					    case 0x28: /* DS18B20 12bit model */
						buffer_tx[ 23 + (17 * j)] = (temp_100s_t % 10) + '0';
						break;
					    case 0x10: /* OLD 1820 model */
						buffer_tx[ 23 + (17 * j)] = '-';
						break;
									}
				    /* normal Tcieczy display */
				    /* always do lookup for WWW */
					if ((temp_100s_t > 8399) && (temp_100s_t < 10000))     {
					    temp_percentage_100s_t = Akas_lookup(temp_100s_t); }

					if ((flagi2 & LIQUID_AS_PERCENT_FLAG) == 0) {
					    memcpy(&virt_lcd_line2[33], &temperature_labels[j][0], 7);
					    memcpy(&virt_lcd_line2[41], &buffer_tx[69], 7);
					    break;
										    }
					else /* liquid percentage display */	{
						if (temp_100s_t < 8400) {
						    virt_lcd_line2[41] = ' ';
						    virt_lcd_line2[42] = '-';
						    virt_lcd_line2[43] = '-';
						    virt_lcd_line2[45] = '-';
						    virt_lcd_line2[46] = '-';
									}
						if (temp_100s_t >= 10000) {
						    virt_lcd_line2[41] = ' ';
						    virt_lcd_line2[42] = ' ';
						    virt_lcd_line2[43] = '0';
						    virt_lcd_line2[45] = '0';
						    virt_lcd_line2[46] = '0';
								 }
						if ((temp_100s_t > 8399) && (temp_100s_t < 10000))  {
						    if (temp_percentage_100s_t <  1000) { virt_lcd_line2[42] = ' '; }
									else 		{ virt_lcd_line2[42] = (temp_percentage_100s_t % 10000) / 1000 + '0'; }
						    virt_lcd_line2[43] = (temp_percentage_100s_t % 1000) / 100 + '0';
						    virt_lcd_line2[45] = ((temp_percentage_100s_t % 100) / 10)+ '0';
						    virt_lcd_line2[46] = (temp_percentage_100s_t % 10) + '0';
												    }
										}
					    break;
						    } /* switch j */

					if (j == 3) { // last reading, finish the line to print
					    memcpy(&buffer_tx[77], "Alarm:-----", 11);
					    if ((alarm_type & PREHEAT_COLUMN_ALARM_TYPE) != 0) { buffer_tx[83] = 'P'; }
					    if ((alarm_type & COLUMN_ALARM_TYPE) != 0) { buffer_tx[83] = 'C'; }
					    if ((alarm_type & HEAD_ALARM_TYPE) != 0) { buffer_tx[83] = 'H'; }
					    if ((alarm_type & TIMER_ALARM_TYPE) != 0) { buffer_tx[86] = 'T'; }
					    if ((alarm_type & WATER_ALARM_TYPE) != 0) { buffer_tx[87] = 'W'; }

					    buffer_tx[88] = ',';
					    memcpy(&buffer_tx[89], "PWM:   ", 7);
					    buffer_tx[96] = '%';
					    if ((flagi3 & PWM_ENABLED_FLAG) == 0) {
						buffer_tx[95] = '-';		  }
					    else {
						if (pwm_output == 100)  {
							buffer_tx[93] = '1';
							buffer_tx[94] = '0';
									}
						else {
						if ((pwm_output / 10) > 0)  {
						    buffer_tx[94] = ((pwm_output / 10) + '0');
									    }
						     }
						buffer_tx[95] = (pwm_output % 10) + '0';
						 }

					    buffer_tx[97] = '\r';
					    buffer_tx[98] = '\n';
					    buffer_tx[99] = 0;

					#if defined(__AVR__)
					    flagi |= FLAG_SERIAL_TX_TO_SEND;  /* buffer_tx to send */
					    tx_pointer = 0;			 /* reset pointer */
					    #if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
						UCSR0B |= _BV(UDRIE0);		 /* ARM Tx buffer empty interrupt */
					    #endif
					    #if defined(__AVR_ATmega8__)
						UCSRB |= _BV(UDRIE);		 /* ARM Tx buffer empty interrupt */
					    #endif
					#endif

					#if defined(__ets__)
					    Serial.print((char *)&buffer_tx[0]);
					    if (IP_connected) { last_TcpClient->sendString((char *)&buffer_tx[0], FALSE); }
					#endif
				    		     } /* if j==3 */

				    } /* switch J*/

			else {  /* error handling */
		#if defined(__ets__)
				temp_100s_valid[j] = false;
		#endif
		#if defined(__AVR__)
				temp_100s_valid &= ~(1 << j);
		#endif

				    if ((j == 0) || (j == 1)) { buffer_tx[(j  * 17) + 9] = 'T'; }
			        else 		      { buffer_tx[(j  * 17) + 9] = ' '; }
				/* copy labels */
				    memcpy(&buffer_tx[(j * 17) + 10],&temperature_labels[j][0], 7);
				    buffer_tx[(j * 17) + 17] = '=';
				    buffer_tx[(j * 17) + 21] = '.';
#if defined(DEFINE_DEGREES_SIGN)
				    buffer_tx[(j * 17) + 24] = 'C';
#else
				    buffer_tx[(j * 17) + 24] = 'C';
#endif
				    buffer_tx[(j * 17) + 25] = ',';

				switch (j) {
				    case 0: /* Tkolumny */
					    buffer_tx[18] = ' ';
					    buffer_tx[19] = '-';
					    buffer_tx[20] = '-';
					    buffer_tx[22] = '-';
					    buffer_tx[23] = '-';
					/* start printing on serial */
					    memcpy(&lcd_line1[0], &buffer_tx[9], 16);
					 break;
				    case 1: 
					    buffer_tx[18 + (17 * j)] = ' ';
					    buffer_tx[19 + (17 * j)] = '-';
					    buffer_tx[20 + (17 * j)] = '-';
					    buffer_tx[22 + (17 * j)] = '-';
					    buffer_tx[23 + (17 * j)] = '-';
					    memcpy(&virt_lcd_line2[(16 * (j - 1))], &buffer_tx[9 + (17 * j)], 16);
					    break;

				/* 3rd line */
				    case 2:
					    buffer_tx[18 + (17 * j)] = ' ';
					    buffer_tx[19 + (17 * j)] = '-';
					    buffer_tx[20 + (17 * j)] = '-';
					    buffer_tx[22 + (17 * j)] = '-';
					    buffer_tx[23 + (17 * j)] = '-';

					    display_3rd_lcd_line();
					break;

				/* 4th line */
				    case 3:
					    buffer_tx[18 + (17 * j)] = ' ';
					    buffer_tx[19 + (17 * j)] = '-';
					    buffer_tx[20 + (17 * j)] = '-';
					    buffer_tx[22 + (17 * j)] = '-';
					    buffer_tx[23 + (17 * j)] = '-';
					    memcpy(&virt_lcd_line2[32], &buffer_tx[60], 16);

					    memcpy(&buffer_tx[77], "Alarm:-----", 11);
					    if ((alarm_type & PREHEAT_COLUMN_ALARM_TYPE) != 0) { buffer_tx[83] = 'P'; }
					    if ((alarm_type & COLUMN_ALARM_TYPE) != 0) { buffer_tx[83] = 'C'; }
					    if ((alarm_type & HEAD_ALARM_TYPE) != 0) { buffer_tx[83] = 'H'; }
					    if ((alarm_type & TIMER_ALARM_TYPE) != 0) { buffer_tx[86] = 'T'; }
					    if ((alarm_type & WATER_ALARM_TYPE) != 0) { buffer_tx[87] = 'W'; }

					    buffer_tx[88] = ',';
					    memcpy(&buffer_tx[89], "PWM:   %", 8);
					    if ((flagi3 & PWM_ENABLED_FLAG) == 0) {
						buffer_tx[95] = '-';		  }
					    else {
						if (pwm_output == 100)  {
						    buffer_tx[93] = '1';
						    buffer_tx[94] = '0';
									}
						else { 
						    if ((pwm_output / 10) > 0) {
							buffer_tx[94] = ((pwm_output / 10) + '0');
									       }
						     }
						buffer_tx[95] = (pwm_output % 10) + '0';
						 }

					    buffer_tx[97] = '\r';
					    buffer_tx[98] = '\n';
					    buffer_tx[99] = 0;

					#if defined(__AVR__)
					    flagi |= FLAG_SERIAL_TX_TO_SEND;  /* buffer_tx to send */
					    tx_pointer = 0;			 /* reset pointer */

					    #if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
						UCSR0B |= _BV(UDRIE0);		 /* ARM Tx buffer empty interrupt */
					    #endif
					    #if defined(__AVR_ATmega8__)
						UCSRB |= _BV(UDRIE);		 /* ARM Tx buffer empty interrupt */
					    #endif
					#endif

					#if defined(__ets__)
					    Serial.print((char *)&buffer_tx[0]);
					    if (IP_connected) { last_TcpClient->sendString((char *)&buffer_tx[0], FALSE); }
					#endif

					break;
					   } /* switch J */

			      } /* /error handling */
						 } /* for J */
	    /* start temperature measuring again */
	    for (j = 0; j < num_of_sensors; j++) {
#if defined(__AVR__)
		    address_dallas(j);
		    ow_byte_wr(DS18X20_CONVERT_T);
#endif
#if defined(__ets__)
		    address_dallas(j, OneWire_bus);
		    OneWire_bus.write(DS18X20_CONVERT_T);
#endif
						 } /* for J */
	    /* vertical scroll */
	    if (lcd_offset < 2) { lcd_offset++; }
		else		{ lcd_offset = 0; }


} // do_1s_stuff

