/*--------------------------------------------------------------*/
/* Sets up Timer 1 to 10ms on AVR, 
 * Timer_1(Timer_1_ISR) to every 10ms on ESP
 * serial port to UART_BAUD on both, 8N1 on AVR, on AVR enables interrupts
 * initializes I/O on AVR (both to be done)			*/
/*--------------------------------------------------------------*/
#include "ioinit.h"

//#ifndef DEFINES_H
    #include "defines.h"
//#endif

#if defined(__ets__)
    #ifndef _NET_WIRING_
	#include <SmingCore/SmingCore.h>
	#include <SmingCore/Platform/WDT.h>
	#include <Libraries/OneWire/OneWire.h>
    #endif
    #if defined(NODEMCU_HARDWARE_TIMER)
	#include <SmingCore/HardwareTimer.h>
    #endif

#endif

#if defined(__AVR__)

#include <avr/io.h>
#include <avr/interrupt.h>

#include "i2c_master.h"

void
Timer1_prescaler_init(void)
{
    switch (Timer1_prescale) {
	case 0	:
	    TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));
	    break;

        case 1	: 
	    TCCR1B |= _BV(CS10);
	    TCCR1B &= ~(_BV(CS11) | _BV(CS12));
	    break;

        case 8	: 
	    TCCR1B |= _BV(CS11);
	    TCCR1B &= ~(_BV(CS10) | _BV(CS12));
	    break;

        case 64 : 
	    TCCR1B |= _BV(CS10) | _BV(CS11);
	    TCCR1B &= ~_BV(CS12);
	    break;

        case 256 : 
	    TCCR1B |= _BV(CS12);
	    TCCR1B &= ~(_BV(CS10) | _BV(CS11));
	    break;
	
        case 1024 : 
	    TCCR1B |= _BV(CS12) | _BV(CS10);
	    TCCR1B &= ~_BV(CS11);
	    break;
	}
}
#endif

/* -------------------------------------------- */

void
ioinit (void)
{
/* outputs setup */

/* -------------------------------------------- */
#if defined(__AVR__)

    #if defined(I2C_IO)
	i2c_init();
//    DDRC |= _BV(PC5);
    #endif

    /*  set LED bits as outputs */
	LED_1_DDR |= _BV(LED_1_BIT);
    #if (defined(PWM_TIMER) && !defined(I2C_IO))
	KEY_5_DDR |= _BV(KEY_5_BIT);
    #endif
    /*  pullups for switches */
#if defined(KEY_1_BIT)
        KEY_1_PORT |= _BV(KEY_1_BIT);
#endif
#if defined(KEY_2_BIT)
        KEY_2_PORT |= _BV(KEY_2_BIT);
#endif
#if defined(KEY_3_BIT)
        KEY_3_PORT |= _BV(KEY_3_BIT);
#endif

#if defined(KEY_4_BIT)
	KEY_4_PORT |= _BV(KEY_4_BIT);
#endif

#if !(defined PWM_TIMER || defined(I2C_IO))
	KEY_5_PORT |= _BV(KEY_5_BIT);
#endif

#if defined(KEY_6_BIT)
	KEY_6_PORT |= _BV(KEY_6_BIT);
#endif

#if defined(ENCODER_PLUS_BIT)
	ENCODER_PLUS_PORT |= _BV(ENCODER_PLUS_BIT);
#endif
#if defined(ENCODER_MINUS_BIT)
	ENCODER_MINUS_PORT |= _BV(ENCODER_MINUS_BIT);
#endif
#if defined(ENCODER_SWITCH_BIT)
	ENCODER_SWITCH_PORT |= _BV(ENCODER_SWITCH_BIT);
#endif
/*--------------------------------------------------------------*/
/* Timer 1 is 16-bit 						*/
/* normal mode, "4", CTC, period in OCR1A, prescaler/64,	*/
/* start timer, this will be 10ms period			*/
/*--------------------------------------------------------------*/
     TCCR1A &= ~(_BV(WGM11) | _BV(WGM10));
     TCCR1B |= _BV(WGM12);
     TCCR1B &= ~_BV(WGM13);

     OCR1A = half_period;

    #if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
	 TIMSK1 |= _BV(OCIE1A);
    #elif defined(__AVR_ATmega8__)
	 TIMSK |= _BV(OCIE1A);
    #else
	#error("Device NOT supported")
    #endif

    Timer1_prescaler_init();

/*------------------------------------------------------*/
/* 	serial port baudrate setup using setbaud.h	*/
/*------------------------------------------------------*/

    #if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
	UBRR0H = UBRRH_VALUE;
	UBRR0L = UBRRL_VALUE;
    #endif
    #if defined(__AVR_ATmega8__)
	UBRRH = UBRRH_VALUE;
        UBRRL = UBRRL_VALUE;
    #endif

    #if USE_2X
	#if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
	    UCSR0A |= (1 << U2X0);
	#endif
	#if defined(__AVR_ATmega8__)
	    UCSRA |= (1 << U2X);
	#endif
    #else
	#if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
	    UCSR0A &= ~(1 << U2X0);
	#endif
	#if defined(__AVR_ATmega8__)
	    UCSRA &= ~(1 << U2X);
	#endif
    #endif

/*------------------------------------------------------*/
/* 8 data bit, 1 stop (default), no parity (default)    */

    #if defined(__AVR_ATmega8__)
	UCSRC = _BV(URSEL) | _BV(UCSZ1) | _BV(UCSZ0);
	UCSRB |= _BV(TXEN);
    #endif
    #if (defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168P__) || defined(__AVR_ATmega88P__))
	UCSR0C = _BV(UCSZ01) | _BV(UCSZ00);
	UCSR0B |= _BV(TXEN0);
    #endif

/*------------------------------------------------------*/
    sei();		/* enable interrupts	 	*/

#endif /* if defined(__AVR__) */

/*--------------------------------------------------------------*/

#if defined(__ets__)
    #if defined(KEY_5_PIN)
	pinMode(KEY_5_PIN, INPUT);
    #endif
    #if defined(KEY_1_PIN)
	pinMode(KEY_1_PIN, INPUT);
    #endif
    #if defined(KEY_2_PIN)
	pinMode(KEY_2_PIN, INPUT);
    #endif
    pinMode(KEY_3_PIN, INPUT_PULLUP);
    pinMode(KEY_4_PIN, INPUT_PULLUP);
//    pinMode(KEY_6_PIN, INPUT_PULLUP);
    pinMode(LED_1_PIN, OUTPUT);

    Timer_1.initializeMs(10, Timer_1_ISR);
    Timer_1.start();
    Timer_2.initializeMs(10, Timer_2_ISR);
    Timer_2.start();
    Timer_500ms.initializeMs(100, Timer_500ms_ISR);
    Timer_500ms.start();

//    spiffs_mount_manual(0x4D000, 0x80000);
    spiffs_mount();

    OneWire_bus.begin();
//    Serial.begin(SERIAL_BAUD_RATE); // 115200 by default
    Serial.begin(UART_BAUD); // 38400 by default
// DISABLE SoftAP
    WifiAccessPoint.enable(false);
//    OneWire ds(OW_PIN);
#endif

}

/*------------------------------------------------------*/
/* END "ioinit.c */
