/*------------------------------------------*/
/* header file for TIMER1 interrupt routine */
/*------------------------------------------*/
#ifndef TIMER_INT_H
    #define TIMER_INT_H
#endif

#if defined(__AVR__)
    #include <avr/interrupt.h>
    #include <avr/eeprom.h>
#endif

#if defined(__ets__)
    #include <SmingCore/Digital.h>
    #include <Libraries/OneWire/OneWire.h>
    #include <Libraries/LiquidCrystal/LiquidCrystal_I2C.h>

    #if defined(HX711_test)
	#include <Libraries/HX711_ESP/HX711.h>
    #endif
#endif

#define uchar uint8_t

extern volatile uint8_t flagi, flagi2, flagi3;
#if defined(__AVR__)
    extern volatile	uint8_t lcd_flagi, lcd_third_line_mode;
#endif
#if defined(__ets__)
    extern		uint8_t	lcd_flagi, lcd_third_line_mode;
#endif

extern volatile uint8_t key_1_filter,key_2_filter,key_3_filter,key_4_filter,key_5_filter;
extern volatile uint8_t key_6_filter,encoder_plus_filter,encoder_minus_filter,encoder_switch_filter,encoder_position;
extern volatile uint8_t u_time, soft_keys, soft_keys2, soft_keys3;
extern uint8_t buzzer_mask, alarm_type;
extern volatile uint8_t soft_key_3_timer, soft_key_4_timer, soft_encoder_switch_timer;

extern const uint8_t 		time_table[TIME_TABLE_MAX][3];

#if defined(__ets__)
    extern		uint8_t configured;
    extern 		bool	IP_connected;
    extern		TcpClient*	last_TcpClient;

    extern		SoftI2cMaster	I2C_bus;
    extern		boolean	STS_Ready;
    extern		float	STS_Temperature;
    extern		float	STS_Temp;

    extern		uint8_t temperature_labels[4][8];
    extern		uint8_t scratchpad_table[4][9];
    extern		uint8_t	i, j, z;
    extern		OneWire	OneWire_bus;
    extern		uint8_t	ds_in_config;
    extern		LiquidCrystal_I2C lcd;
    extern		uint8_t	num_of_sensors;
    extern		uint8_t	sensor_addreses[4][8];
    extern		uint8_t wait_key_release;
    extern		uint8_t	lcd_offset;
    /* deltas */
    extern		uint8_t		temp_sign_d;
    extern		uint16_t	temp_100s_d;
    extern		uint8_t		temp_units[4];
    extern		uint16_t	temp_parts[4];
    extern		uint8_t		temp_sign[4];

    extern		uint16_t	temp_100s[4]; /* temperature as 1/100 degrees, for DELTA calculations  */
    extern		uint16_t	temp_100s_delta[4]; /* temperature as 1/100 degrees, for DELTA calculations  */
    extern		bool		temp_100s_delta_sign[4]; /* temperature as 1/100 degrees, for DELTA calculations  */
#if defined(__ets__)
    extern		boolean		temp_100s_valid[4];
#endif
    /* temporary storage */
    extern		uint8_t		temp_units_t;
    extern		uint16_t	temp_parts_t;
    extern		uint8_t		temp_sign_t;
    extern		uint16_t	temp_100s_t;

/* preheat alarm calculations */
    extern		uint32_t	temp_100s_t2;
    extern		uint32_t	temp_100s_t2_max;
    extern		uint32_t	temp_100s_t2_min;
    extern		uint8_t		buffer_tx[120];
    extern volatile	uint8_t		error_code;
    extern		uint8_t		lcd_time[10];
    extern		uint16_t	temp_1_filter_10sec_100s[10];
    extern		uint8_t		temp_1_filter_10sec_pointer;
    extern		uint16_t	temp_1_filtered_10sec_100s;
    extern		uint16_t	temp_2_filtered_10sec_100s;
    extern		uint32_t	temp_1_filtered_10sec_100s_32;
    extern		uint32_t	temp_2_filtered_10sec_100s_32;

#endif

extern uint16_t	temp_delta_base_100s[4];
extern uint16_t	temp_100s[4];
extern char	lcd_line1[17];
extern uint8_t	virt_lcd_line2[49];

#if defined(HEARTBEAT_LED) && defined(__ets__)
    extern bool	state;
#endif

#if defined(__AVR__)
    extern const uint8_t lcd_flags_in_eeprom EEMEM;
#endif

extern uint8_t ones_count8(uint8_t code, uint8_t bits);

#if defined(INCLUDE_RTC)
    extern volatile uint8_t hour, minute, second;
#endif

/* ---------------------------------------------------------------------- */

#if defined(DOWNCOUNT_TIMER)
    extern volatile uint8_t downcount_timer_hour, downcount_timer_minute, downcount_timer_second;
    #if defined(__AVR__)
	extern 	const	uint8_t	time_table[TIME_TABLE_MAX][3] PROGMEM;
    #endif
    #if defined(__ets__)
	extern 	const	uint8_t	time_table[TIME_TABLE_MAX][3];
    #endif

#endif

/* ---------------------------------------------------------------------- */

#if defined(AUTO_PWM_DOWNCOUNT_TIMER)
    extern volatile uint8_t auto_pwm_downcount_timer_minute, auto_pwm_downcount_timer_second;
//    #if defined(__AVR__)
//	extern 	const	uint8_t	time_table[TIME_TABLE_MAX][3] PROGMEM;
//    #endif
//    #if defined(__ets__)
//	extern 	const	uint8_t	time_table[TIME_TABLE_MAX][3];
//    #endif
#endif

/* ---------------------------------------------------------------------- */

#if defined(PWM_TIMER)
extern volatile uint8_t pwm_output_idx, pwm_timer, pwm_output;
extern const	uint8_t PWM_table[PWM_TABLE_MAX] PROGMEM;
#endif

#if defined(__AVR__)
    extern ISR(TIMER1_COMPA_vect);
#endif
#if defined(__ets__)
    extern void Timer_1_ISR(void);
//    extern void Timer_500ms_ISR(void);
    extern void prepare_time_to_display(void);
#endif

/*------------------------------------------*/
